import { useState, useEffect, useRef, useMemo } from 'react';
import { useSelector } from 'react-redux';

import { TextField, Grid, Typography } from '@material-ui/core';
import { Autocomplete } from '@material-ui/lab';
import { LocationOn } from '@material-ui/icons';

import { makeStyles } from '@material-ui/core/styles';
import parse from 'autosuggest-highlight/parse';
import throttle from 'lodash/throttle';

import { transformToCoords } from '../../utils/getCoordinates';
import PropTypes from 'prop-types';
require('dotenv').config();

const googleApiKey = process.env.REACT_APP_GOOGLE_API_KEY;


function loadScript(src, position, id) {
  if (!position) {
    return;
  }

  const script = document.createElement('script');
  script.setAttribute('async', '');
  script.setAttribute('id', id);
  script.src = src;
  position.appendChild(script);
};

const autocompleteService = { current: null };

const useStyles = makeStyles((theme) => ({
  icon: {
    color: theme.palette.text.secondary,
    marginRight: theme.spacing(2),
  },
  input: {
    marginTop: 10
  }
}));

export const GoogleAutocompletePlace = ({ setPlace, setLongPlace, setCoords, errors }) => {
  const classes = useStyles();
  const [value, setValue] = useState(null);
  const [inputValue, setInputValue] = useState('');
  const [options, setOptions] = useState([]);
  const loaded = useRef(false);
  const placeFromSelector = useSelector(state => state.selectedActivity);
  const place = "activity" in placeFromSelector ? placeFromSelector.activity.longPlace : '';

  useEffect(() => {
    setInputValue(value ? value : place);
    //eslint-disable-next-line
  }, []);

  useEffect(() => {
    if (value !== null) {
      transformToCoords(value.description, setCoords);
      setPlace(value.structured_formatting.main_text);
      setLongPlace(value.description);
    }
    //eslint-disable-next-line
  }, [value]);

  if (typeof window !== 'undefined' && !loaded.current) {
    if (!document.querySelector('#google-maps')) {
      loadScript(
        `https://maps.googleapis.com/maps/api/js?key=${googleApiKey}&libraries=places`,
        document.querySelector('head'),
        'google-maps',
      );
    }

    loaded.current = true;
  }

  const fetch = useMemo(
    () =>
      throttle((request, callback) => {
        autocompleteService.current.getPlacePredictions(request, callback);
      }, 200),
    [],
  );

  useEffect(() => {
    let active = true;

    if (!autocompleteService.current && window.google) {
      autocompleteService.current = new window.google.maps.places.AutocompleteService();
    }
    if (!autocompleteService.current) {
      return undefined;
    }

    if (inputValue === '') {
      setOptions(value ? [value] : []);
      return undefined;
    }

    fetch({ input: inputValue }, (results) => {
      if (active) {
        let newOptions = [];

        if (value) {
          newOptions = [value];
        }

        if (results) {
          newOptions = [...newOptions, ...results];
        }

        setOptions(newOptions);
      }
    });

    return () => {
      active = false;
    };
  }, [value, inputValue, fetch]);

  return (
    <Autocomplete
      id="google-map-demo"
      getOptionLabel={(option) => (typeof option === 'string' ? option : option.description)}
      filterOptions={(x) => x}
      options={ options }
      autoComplete
      includeInputInList
      filterSelectedOptions
      value={ value }
      color="secondary"
      fullWidth
      onChange={(event, newValue) => {
        setOptions(newValue ? [newValue, ...options] : options);
        setValue(newValue);
      }}
      onInputChange={(event, newInputValue) => {
        console.log('newInputValue', newInputValue);
        setInputValue(newInputValue);
      }}
      renderInput={(params) => (
        <TextField
          {...params}
          label="Punt de trobada *"
          variant="outlined"
          fullWidth
          className={ classes.input }
          color="secondary"
          error={ errors.place !== undefined }
          helperText={ errors.place }
        />
      )}
      renderOption={(option) => {
        const matches = option.structured_formatting.main_text_matched_substrings;
        const parts = parse(
          option.structured_formatting.main_text,
          matches.map((match) => [match.offset, match.offset + match.length]),
        );

        return (
          <Grid container alignItems="center">
            <Grid item>
              <LocationOn className={ classes.icon } />
            </Grid>
            <Grid item xs>
              { parts.map((part, index) => (
                <span key={ index } style={{ fontWeight: part.highlight ? 700 : 400 }}>
                  { part.text }
                </span>
              ))}

              <Typography variant="body2" color="textSecondary">
                { option.structured_formatting.secondary_text }
              </Typography>
            </Grid>
          </Grid>
        );
      }}
    />
  );
};

GoogleAutocompletePlace.propTypes = {
  setPlace: PropTypes.func,
  setLongPlace: PropTypes.func,
  setCoords: PropTypes.func,
  errors: PropTypes.object,
};
