import { useState, useEffect } from 'react';
import { useParams } from 'react-router-dom';
import { useSelector, useDispatch } from 'react-redux';
import axios from 'axios';
import { setActivityDetails } from '../redux/actions/activityActions';
import { setActivityCoords } from '../redux/actions/coordsActions';
import { fetchCommentariesByActivityId } from '../redux/actions/commentariesActions';

import { SortedActivities } from './SortedActivities';
import { TitleSection } from '../components/styled/TitleSection';
import { FloatingButton, FloatingButtonGoBack } from '../components/units/Button';
import { categories } from '../utils/categories';
import {
  coloured, InfoBox, Icon, InfoItems, MapDescContainer, CardBox, numParticipantsBox, infoIcon, LoadingDiv, WhiteSection, emojiStyles
} from './ActivityDetails.styles';
import { SuccessMessage } from '../components/composed/SuccessMessage';
import { ReactMap } from '../components/composed/ReactMap';
import { ParticipantBox } from '../components/composed/ParticipantBox';

import { Container, CardContent, CircularProgress, Link } from "@material-ui/core";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { Place, EventAvailable, AccessTime, EmojiPeople } from '@material-ui/icons';
import { Breadcrumb } from '../components/units/Breadcrumb';

import { formateDate } from '../utils/formateDate';
import { Commentaries } from "../components/composed/Commentaries";
import { EditActivity } from '../components/composed/EditActivity';
import { ParticipateButton } from '../components/composed/ParticipateButton';


export const ActivityDetails = () => {
  const dispatch = useDispatch();
  const id = useParams().activityId;
  const userId = useSelector(state => state.user.user._id);
  const [activity, setActivity] = useState([]);
  const [isLoading, setIsLoading] = useState(false);
  const [saved, setSaved] = useState(false);
  const [isParticipating, setIsParticipating] = useState(null);
  const [icon, setIcon] = useState('');
  const [date, setDate] = useState('');
  const [participants, setParticipants] = useState(0);
  const [showParticipantsBox, setShowParticipantsBox] = useState(false);
  const [allParticipants, setAllParticipants] = useState([]);
  const [coords, setCoords] = useState({
    lat: 41.40376668439598,
    lng: 2.1743987132017897,
  });
  const icons = categories;
  const textSuccessMessage = isParticipating ? "T'has apuntat a l'activitat" : "T'has desapuntat";

  const handleOnMouseOver = () => setShowParticipantsBox(true);

  console.log('coords', coords);

  useEffect(() => {
    const signal = axios.CancelToken.source();
    setIsLoading(true);
    const url = `https://entrenem.herokuapp.com/api/activities/${id}`;

    axios.get(url, {
      cancelToken: signal.token
    })
      .then(response => {
        const data = response.data[0];
        dispatch(setActivityDetails(data));
        setActivity(data);
        setIcon(icons.filter(element => element.title === data.category));
        setDate(formateDate(data.date, false));

        const activityCoords = {
          lat: parseFloat(data.lat),
          lng: parseFloat(data.lng)
        };
        setCoords(activityCoords);
        dispatch(setActivityCoords(activityCoords));
        if (data.participants !== null && data.participants[0] !== "") {
          setParticipants(data.participants.length);
          setAllParticipants(data.participants);
          setIsParticipating(data.participants.some(el => el.userId === userId) ? true : false);
        }
        setIsLoading(false);
      })
      .catch(error => console.log(error));

    dispatch(fetchCommentariesByActivityId(id));

    return function cleanup() {
      signal.cancel();
    };
    // eslint-disable-next-line
  }, [id]);

  // useEffect(() => {
  //   setUsersParticipating(allParticipants);
  // }, [allParticipants]);

  return (
    <>
      { isLoading && <LoadingDiv><CircularProgress color="secondary" /></LoadingDiv> }
      { !isLoading && <>
        <Breadcrumb text="activitat" title={ activity.title } />
        { activity.createdBy === userId && <EditActivity activityId={ id }/> }
        <FloatingButton />
        <FloatingButtonGoBack />
        <section style={ coloured }>
          <InfoBox>
            { icon.length > 0 && <Icon><FontAwesomeIcon icon={ icon[0].icon } /></Icon> }
            <TitleSection>{ activity.title }</TitleSection>
            <InfoItems>
              <div>
                <Place style={ infoIcon } />
                <span>{ activity.place }</span>
              </div>
              <div>
                <EventAvailable style={ infoIcon } />
                <span>{ date }</span>
              </div>
              <div>
                <AccessTime style={ infoIcon } />
                <span>{ activity.time }</span>
              </div>
            </InfoItems>
            { activity.createdBy !== userId && <ParticipateButton
              participate={ isParticipating }
              setParticipate={ setIsParticipating }
              participants={ participants }
              setParticipants={ setParticipants }
              allParticipants={ allParticipants }
              setAllParticipants={ setAllParticipants }
              setSaved={ setSaved }
              activityId={ id }
            />
            }
          </InfoBox>
        </section>
        <WhiteSection>
          <MapDescContainer maxWidth="lg">
            <ReactMap />
            <CardBox>
              <CardContent>
                <p>{ activity.description }</p>
              </CardContent>
            </CardBox>
            <div style={ numParticipantsBox }>
              <EmojiPeople style={ emojiStyles } />
              <span onMouseOver={ () => handleOnMouseOver() }><Link>{ participants } usuaris inscrits</Link></span>
              <span> - - activitat creada per { activity.createdByName }</span>
              { participants > 0 && showParticipantsBox && <ParticipantBox
                participants={ allParticipants }
                setShowParticipantsBox={ setShowParticipantsBox }
              /> }
            </div>
          </MapDescContainer>
        </WhiteSection>
        <section style={ coloured }>
          <Container maxWidth="lg">
            <TitleSection>Comentaris</TitleSection>
            <Commentaries activityId={ id } />
          </Container>
        </section>
        <section>
          <SortedActivities show="relacionades" title="Activitats relacionades" category={ activity.category } topPosition />
        </section>
        </>
      }
      { saved && <SuccessMessage text={ textSuccessMessage } style={{ zIndex: 3 }} /> }
    </>
  );
};
