import { useState, useEffect } from 'react';
import { useHistory } from 'react-router-dom';
import { useSelector, useDispatch } from 'react-redux';
import { setModifiedActivity, setNewActivity } from '../redux/actions/activityActions';
import { setActivityCoords } from '../redux/actions/coordsActions';

import logoIcon from '../assets/logo/logo-icon.svg';
import { Container, Select, TextField, FormControl, InputLabel, MenuItem } from "@material-ui/core";
import { TitleSection } from '../components/styled/TitleSection';
import {
  GridContainer, TextBoxField, coloured, SizeContainer, DescriptionTitle, ButtonContainer, ErrorSpan, IconContainer, logoStyles,
} from './NewActivity.styles';
import { SuccessMessage } from '../components/composed/SuccessMessage';
import { Breadcrumb } from '../components/units/Breadcrumb';
import { ContainedButton, FloatingButtonGoBack, OutlinedButton } from '../components/units/Button';

import { GoogleAutocompletePlace } from '../components/composed/GoogleAutocompletePlace';
import { ReactMap } from '../components/composed/ReactMap';

export const NewActivity = () => {
  const dispatch = useDispatch();
  const history = useHistory();
  const userId = useSelector(state => state.user.user._id);
  const userName = useSelector(state => state.user.user.fullName);
  const activity = useSelector(state => state.selectedActivity.activity);
  const isNew = useSelector(state => state.isNew);
  const [category, setCategory] = useState('');
  const [title, setTitle] = useState('');
  const [date, setDate] = useState('');
  const [time, setTime] = useState('');
  const [place, setPlace] = useState('');
  const [longPlace, setLongPlace] = useState('');
  const [description, setDescription] = useState('');
  const [coords, setCoords] = useState('');
  const [saved, setSaved] = useState(false);
  const [errors, setErrors] = useState({});
  const [textBreadcrumb, setTextBreadcrumb] = useState('nova-activitat');

  const handleClick = () => {
    console.log('isNew', isNew);

    const newActivity = {
      category,
      title,
      date,
      time,
      place,
      longPlace,
      description,
      createdBy: userId,
      createdByName: userName,
      participants: [],
      lat: coords.lat,
      lng: coords.lng,
    };

    isNew && dispatch(setNewActivity(newActivity, userId, userName, history, setSaved, setErrors));
    !isNew && activity !== undefined && dispatch(setModifiedActivity(newActivity, activity._id, userId, userName, history, setSaved, setErrors));
  };

  useEffect(() => {
    if (activity && !isNew) {
      setCategory(activity.category);
      setTitle(activity.title);
      setDate(activity.date);
      setTime(activity.time);
      setPlace(activity.longPlace);
      setDescription(activity.description);
      setCoords({ lat: activity.lat, lng: activity.lng });
      dispatch(setActivityCoords(coords));
    }

    !isNew && setTextBreadcrumb('modifica-activitat');
    //eslint-disable-next-line
  }, []);

  useEffect(() => {
    dispatch(setActivityCoords(coords));
    //eslint-disable-next-line
  }, [coords]);

  return (
    <section style={ coloured }>
      <Breadcrumb text={ textBreadcrumb } />
      <FloatingButtonGoBack />
      <Container maxWidth="lg">
        <IconContainer>
          <img src={ logoIcon } alt="Icon del logo d'Entrenem" style={ logoStyles } />
        </IconContainer>
        <TitleSection>{ isNew ? 'Nova Activitat' : 'Modifica l\'activitat' }</TitleSection>
        <form noValidate autoComplete="off">

          <GridContainer>
            <div>
              <FormControl variant="outlined" fullWidth color="secondary" required>
                <InputLabel id="selectCat">Categoria</InputLabel>
                <Select
                  labelId="selectCat"
                  id="selectCat"
                  value={ category }
                  onChange={(e) => setCategory(e.target.value)}
                  label="Categoria *"
                  color="secondary"
                  required
                  error={ errors.category !== undefined }
                >
                  <MenuItem value="bicicleta">Bicicleta</MenuItem>
                  <MenuItem value="caminar">Caminar</MenuItem>
                  <MenuItem value="e.funcional">Entrenament funcional</MenuItem>
                  <MenuItem value="escalar">Escalar</MenuItem>
                  <MenuItem value="ioga">Ioga</MenuItem>
                  <MenuItem value="natacio">Natació</MenuItem>
                  <MenuItem value="patinatge">Patinatge</MenuItem>
                  <MenuItem value="running">Running</MenuItem>
                  <MenuItem value="senderisme">Senderisme</MenuItem>
                  <MenuItem value="tennis taula">Tennis taula</MenuItem>
                  <MenuItem value="volei platja">Vòlei platja</MenuItem>
                </Select>
              </FormControl>
              { errors.category && <ErrorSpan variant="caption">{ errors.category }</ErrorSpan>}

              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                id="title"
                label="Títol de l'activitat"
                name="email"
                color="secondary"
                value={ title }
                onChange={(e) => setTitle(e.target.value)}
                error={ errors.title !== undefined }
                helperText={ errors.title }
              />

              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                id="date"
                label="Dia"
                type="date"
                color="secondary"
                value={ date }
                onChange={(e) => setDate(e.target.value)}
                InputLabelProps={{ shrink: true, }}
                error={ errors.date !== undefined }
                helperText={ errors.date }
              />

              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                id="time"
                label="Hora"
                type="time"
                color="secondary"
                value={ time }
                onChange={(e) => setTime(e.target.value)}
                error={ errors.time !== undefined }
                helperText={ errors.time }
                InputLabelProps={{
                  shrink: true,
                }}
              />
              <GoogleAutocompletePlace
                setPlace={ setPlace }
                setLongPlace={ setLongPlace }
                setCoords={ setCoords }
                errors={ errors }
                isNew
              />
            </div>

            <div>
              <ReactMap />
            </div>
          </GridContainer>

          <SizeContainer>
            <DescriptionTitle>Què han de saber els usuaris?</DescriptionTitle>
            <TextBoxField
              variant="outlined"
              margin="normal"
              required
              fullWidth
              id="description"
              label="Afegeix una descripció o comentari"
              type="text"
              multiline
              color="secondary"
              value={ description }
              onChange={(e) => setDescription(e.target.value)}
              error={ errors.description !== undefined }
              helperText={ errors.description }
            />

            <ButtonContainer>
              <ContainedButton text="Guardar" onClick={() => handleClick() } />
              <OutlinedButton text="Cancel·lar" onClick={() => history.goBack()} />
            </ButtonContainer>
          </SizeContainer>
        </form>

      </Container>
      { saved && <SuccessMessage text="S'ha guardat l'activitat" /> }
    </section>
  );
};
