import { useState } from 'react';
import { useHistory } from 'react-router-dom';
import { useDispatch } from 'react-redux';
import axios from 'axios';
import { getCommentedActivities } from '../redux/actions/activitiesActions';

import {
  TextField, Checkbox, Container, InputAdornment, IconButton, CircularProgress, Link
} from '@material-ui/core';
import { Lock, Visibility, VisibilityOff } from '@material-ui/icons';
import { ContainedButton, LinkOutlinedButton } from '../components/units/Button';
import { TitleSection } from '../components/styled/TitleSection';
import { DivBox, Icon, Label, checkLabel, DividerLine, GridDiv, flexBox, btn, CheckboxErrorParagraph } from './Login.Signup.styles';
import { SuccessMessage } from '../components/composed/SuccessMessage';
import { Copyright } from '../components/composed/Copyright';

export const Signup = () => {
  const dispatch = useDispatch();
  const signal = axios.CancelToken.source();
  const history = useHistory();
  const [fullName, setFullName] = useState('');
  const [email, setEmail] = useState('');
  const [password, setPassword] = useState('');
  const [showPassword, setShowPassword] = useState(false);
  const [errors, setErrors] = useState({});
  const [saved, setSaved] = useState(false);
  const [isLoading, setIsLoading] = useState(false);
  const [checked, setChecked] = useState(true);
  const [checkboxError, setCheckboxError] = useState(false);

  const handlePasswordVisibility = () => {
    setShowPassword(!showPassword);
  };

  const handleMouseDownPassword = (e) => {
    e.preventDefault();
  };

  const handleChange = (e) => {
    setChecked(e.target.checked);
  };

  const handleSubmit = (e) => {
    e.preventDefault();
    setErrors({});
    setIsLoading(true);
    setCheckboxError(false);
    console.log('checkboxError', checkboxError);

    if (checked) {
      // API Call with axios
      const user = {
        'fullName': fullName,
        'email': email,
        'password': password
      };

      const url = 'https://entrenem.herokuapp.com/api/users/registrar';
      const config = {
        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
      };

      const params = new URLSearchParams()
      params.append('fullName', user.fullName);
      params.append('email', user.email);
      params.append('password', user.password);

      axios.post(url, params, config, {
        cancelToken: signal.token
      })
        .then((response) => {
          setIsLoading(false);
          setSaved(true);
          dispatch(getCommentedActivities([]));
          setTimeout(() => {
            setSaved(false);
            history.push("/iniciar");
          }, 2000);
        })
        .catch(error => {
          if (error.response.status === 400) {
            setErrors(error.response.data);
            setIsLoading(false);
            console.log('error response data', error.response.data);
          }
        });
    } else {
      setCheckboxError(true);
      setIsLoading(false);
    }

  };

  return (
    <>
      <DivBox style={{ paddingBottom: 30 }}>
        <Container maxWidth="xs">
          <div style={ flexBox }>
            <Icon><Lock /></Icon>
            <TitleSection>Registra't</TitleSection>
            <form noValidate autoComplete="off">
              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                id="fullName"
                label="Nom d'usuari"
                name="fullName"
                color="secondary"
                value={ fullName }
                onChange={(e) => setFullName(e.target.value)}
                error={ errors.fullName !== undefined }
                helperText={ errors.fullName }
              />
              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                id="email"
                label="Correu electrònic"
                name="email"
                autoComplete="email"
                color="secondary"
                value={ email }
                onChange={(e) => setEmail(e.target.value)}
                error={ errors.email !== undefined }
                helperText={ errors.email }
              />
              <TextField
                variant="outlined"
                margin="normal"
                required
                fullWidth
                name="password"
                label="Paraula de pas"
                type={ showPassword ? 'text' : 'password' }
                value={ password.password }
                onChange={(e) => setPassword(e.target.value)}
                id="password"
                color="secondary"
                error={ errors.password !== undefined }
                helperText={ errors.password }
                InputProps={{
                  endAdornment: (
                    <InputAdornment position='end'>
                      <IconButton
                        aria-label='toggle password visibility'
                        onClick={ handlePasswordVisibility }
                        onMouseDown={ handleMouseDownPassword }
                      >
                        { showPassword ? <Visibility /> : <VisibilityOff /> }
                      </IconButton>
                    </InputAdornment>
                  ),
                }}
              />
              <Label
                control={<Checkbox value="accept" color="secondary" />}
                label={
                    <span style={ checkLabel }>Accepto les
                      <Link href="/avis-legal"> Avís legal </Link>i la
                      <Link href="/politica-de-privacitat"> Política de privacitat </Link>d'entrenem
                  </span>
                }
                checked={ checked }
                onChange={ handleChange }
              />
              { checkboxError && <CheckboxErrorParagraph>Has d'acceptar les Condicions del servei i la Política de privacitat per poder registrar-te a entrenem</CheckboxErrorParagraph> }
              <ContainedButton
                type="submit"
                text="Registra't"
                startIcon={ isLoading && <CircularProgress color="primary" /> }
                onClick={ handleSubmit }
              />
              <DividerLine />
              <GridDiv>
                <span>Ja tens compte?</span>
                <LinkOutlinedButton text="Inicia sessió" path="/iniciar" style={ btn } />
              </GridDiv>
            </form>
          </div>
          <Copyright  />
        </Container>
        { saved && <SuccessMessage text="S'ha guardat l'usuari" /> }
      </DivBox>
    </>
  );
};
