import { useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useLocation } from 'react-router-dom';
import PropTypes from 'prop-types';
import { fetchAllActivities, fetchCategoryActivities, fetchCommentedActivities, fetchParticipateActivities, fetchProposedActivities } from '../redux/actions/activitiesActions';
import { activityIsNew } from '../redux/actions/index';

import { loadingStyles, paragraph, paragraphWhite } from './SortedActivities.styles';
import { ActivitiesList } from '../components/composed/ActivitiesList';
import { TitleSection } from '../components/styled/TitleSection';
import { FloatingButton, FloatingButtonGoBack } from '../components/units/Button';
import { Container, CircularProgress } from '@material-ui/core';
import { theme } from '../assets/theme/theme';
import { Breadcrumb } from '../components/units/Breadcrumb';
import logoIcon from '../assets/logo/logo-icon.svg';

export const SortedActivities = ({ show, title, color, category, extraStyles, topPosition }) => {
  const dispatch = useDispatch();
  const location = useLocation();
  const localStorageUser = JSON.parse(localStorage.getItem('entrenemUser'));
  const storeUserId = useSelector(state => state.user.user._id);
  const userId = localStorageUser !== null ? localStorageUser._id : storeUserId;
  const isLoading = useSelector(state => state.isLoading);
  const activities = useSelector(state => state.activities.activities);
  const proposedActivities = useSelector(state => state.activities.proposedActivities);
  const participateActivities = useSelector(state => state.activities.participateActivities);
  const commentedActivitiesFromStore = useSelector(state => state.activities.commentedActivities);
  const [commentedActivities, setCommentedActivities] = useState(commentedActivitiesFromStore);
  const categoryActivities = useSelector(state => state.activities.categoryActivities);
  const nextActivities = activities.length > 0 ? activities.slice(0, 12) : [];

  const sortBy = show;
  const bgColor = color ? theme.palette.primary.light : 'white';
  const additionalStyles = extraStyles ? extraStyles : '';

  const containerStyles = {
    position: 'relative',
    height: '100%',
    marginTop: 0,
    backgroundColor: bgColor,
    additionalStyles,
  };

  const normalPositioning = {
    width: 50,
    marginTop: 50,
    marginBottom: -20,
    marginLeft: 'calc(50vw - 16px - 25px)',

    [theme.breakpoints.up('md')]: {
      left: 'calc(50vw - 24px - 25px)',
    }
  };

  const absolutePositioning = {
    position: 'absolute',
    left: 'calc(50vw - 16px)',
    top: -26,
    width: 50,

    [theme.breakpoints.up('md')]: {
      left: 'calc(50vw - 24px)',
    }
  };

  const logoStyles = topPosition ? absolutePositioning : normalPositioning;

  useEffect(() => {
    sortBy === 'totes' && dispatch(fetchAllActivities());
    sortBy === 'properes' && dispatch(fetchAllActivities());
    sortBy === 'proposades' && dispatch(fetchProposedActivities(userId));
    sortBy === 'inscrites' && dispatch(fetchParticipateActivities(userId));
    sortBy === 'comentades' && dispatch(fetchCommentedActivities(userId));
    sortBy === 'categoria' && dispatch(fetchCategoryActivities(category));
    sortBy === 'relacionades' && dispatch(fetchCategoryActivities(category));

    dispatch(activityIsNew());

    //eslint-disable-next-line
  }, []);

  useEffect(() => {
    setCommentedActivities(commentedActivitiesFromStore);
  }, [commentedActivitiesFromStore]);

  return (
    <Container maxWidth="xl" style={ containerStyles }>
      { sortBy !== 'relacionades' && location.pathname !== '/activitats' && <Breadcrumb text={ sortBy } /> }
      <div style={{ marginLeft: -22, paddingTop: 0 }}>
        { sortBy !== 'relacionades' && location.pathname !== '/activitats' && <FloatingButtonGoBack /> }
        </div>
      <FloatingButton />
      <img src={ logoIcon } alt="Icon del logo d'Entrenem" style={ logoStyles } />
      <TitleSection>{ title }</TitleSection>
      { isLoading && <div style={ loadingStyles }>
        <CircularProgress color="secondary" />
      </div> }

      { !isLoading && sortBy === 'proposades' && proposedActivities.length > 0 &&
        <ActivitiesList activities={ proposedActivities } />
      }
      { !isLoading && sortBy === 'proposades' && proposedActivities.length < 1 && <p style={ paragraph }>Encara no has proposat cap activitat</p> }


      { !isLoading && sortBy === 'inscrites' && participateActivities !== undefined &&
        <ActivitiesList activities={ participateActivities } />
      }
      { !isLoading && sortBy === 'inscrites' && participateActivities.length < 1 && <p style={ paragraphWhite }>No has elegit cap activitat per a participar-hi</p> }


      { !isLoading && sortBy === 'categoria' && categoryActivities.length > 0 &&
        <ActivitiesList activities={ categoryActivities } />
      }
      { !isLoading && sortBy === 'categoria' && categoryActivities.length < 1 && <p style={ paragraph }>Encara no hi ha cap  activitat d'aquesta categoria</p> }


      { !isLoading && sortBy === 'relacionades' && categoryActivities.length > 0 &&
        <ActivitiesList activities={ categoryActivities } />
      }
      { !isLoading && sortBy === 'relacionades' && categoryActivities.length < 1 && <p style={ paragraph }>Encara no hi ha cap  activitat d'aquesta categoria</p> }


      { !isLoading && sortBy === 'comentades' && commentedActivities.length > 0 &&
        <ActivitiesList activities={ commentedActivities } />
      }
      { !isLoading && sortBy === 'comentades' && commentedActivities.length < 1 && <p style={ paragraph }>Encara no has comentat cap activitat</p> }


      { !isLoading && sortBy === 'totes' && activities.length > 0 && <ActivitiesList activities={ activities } /> }

      { !isLoading && sortBy === 'properes' && nextActivities.length > 0 &&
        <ActivitiesList activities={ nextActivities } />
      }
    </Container>
  );
};

SortedActivities.propTypes = {
  show: PropTypes.string,
  title: PropTypes.string,
  color: PropTypes.bool,
  category: PropTypes.string,
  extraStyles: PropTypes.object,
  topPosition: PropTypes.bool,
};
